#!/usr/bin/python
# Lifesaver by Chris Jones <cmsj@tenshu.net>
# GPL v2 only
"""goorender.py - Renderer that uses GooCanvas"""

from __future__ import division
import gtk
import glib
import goocanvas
import pango
import rsvg
from util import DEBUG,dbg
from lifesaver import Lifesaver
from svg import CustomSvgItem

logo_handle = rsvg.Handle(
        '/usr/share/icons/Human/scalable/places/distributor-logo.svg')

class GooRender(goocanvas.Canvas):
    # pylint: disable-msg=R0904
    # pylint: disable-msg=R0902
    """Class that implements a GooCanvas renderer"""
    lifesaver = None
    root = None
    grouproot = None
    postroot = None

    goosource = None
    goouser = None
    goodate = None
    gootext = None

    goodebug = None
    counter = None
    handler = None

    data = None

    def __init__(self):
        """Class initialiser"""
        goocanvas.Canvas.__init__(self)
        self.lifesaver = Lifesaver()
        config = self.lifesaver.config
        self.data = {}

        self.set_property('background-color', config['background_color'])

    def setup(self, hook = None):
        """Prepare the canvas for later use"""
        dbg('GooRender', 'setup')
        data = self.data

        config = self.lifesaver.config
        self.root = self.get_root_item()

        sourcetxt = ''
        usertxt = ''
        datetxt = ''
        texttxt = ''

        self.counter = 0

        # Precalculate co-ordinates and sizes for as many elements as possible
        # Start with anything we need to query
        (data['screenwidth'], data['screenheight']) = self.window.get_size()

        # Distances/sizes which we are hard-coding
        data['screenmargin'] = 20
        data['postmargin'] = 10
        data['authorheight'] = 45
        data['dateheight'] = 23
        data['logoheight'] = 128 # We should really read image sizes out of the SVGItem
        data['logowidth'] = 128  # ditto
        data['postwidth'] = min(800, data['screenwidth'] - 160)
        data['postheight'] = 600 - data['logoheight']

        # Calculate how much we should scale the post
        data['idealheight'] = data['screenheight'] - data['logoheight'] - (data['screenmargin'] * 2)
        data['idealwidth'] = data['screenwidth'] - (data['screenmargin'] * 2)
        data['vertscale'] = data['idealheight'] / data['postheight']
        data['horizscale'] = data['idealwidth'] / data['postwidth']
        data['postscale'] = min(data['vertscale'], data['horizscale'])

        # Calculate what we consider to be off-screen
        data['leftwing'] = -(data['screenwidth'] / 2) - (data['postwidth'] / 2)
        data['rightwing'] = (data['screenwidth'] / 2) + (data['postwidth'] / 2)

        # Position of the post (dimensions are hard coded above)
        data['postx'] = (data['screenwidth'] / 2) - (data['postwidth'] / 2)
        data['posty'] = (data['screenheight'] / 2) - (data['postheight'] / 2)

        # Anchor, position and width of the author text (height is hard coded
        # above)
        data['authoranchor'] = gtk.ANCHOR_NW
        data['authorx'] = data['postmargin']
        data['authory'] = data['postmargin']
        data['authorwidth'] = data['postwidth'] - (data['postmargin'] * 2)

        # Anchor, position and width of the date text (height is hard coded
        # above
        data['dateanchor'] = gtk.ANCHOR_NW
        data['datex'] = data['postmargin']
        data['datey'] = data['authory'] + data['authorheight']
        data['datewidth'] = data['postwidth'] - (data['postmargin'] * 2)

        # Anchor and position of the post's text (width and height are
        # calculated above
        data['textanchor'] = gtk.ANCHOR_NW
        data['textx'] = data['postmargin']
        data['texty'] = data['postmargin'] + data['authorheight'] + data['dateheight']
        data['textwidth'] = data['postwidth'] - (data['postmargin'] * 2)
        data['textheight'] = data['postheight'] - data['authorheight'] - data['dateheight'] - (data['postmargin'] * 2)

        # Position of the logo SVG
        data['logox'] = data['screenmargin']
        data['logoy'] = data['screenheight'] - data['screenmargin'] - data['logoheight']

        # Anchor, position and size of the source text
        data['sourceanchor'] = gtk.ANCHOR_W
        data['sourcex'] = data['screenmargin'] + data['logowidth'] + data['postmargin']
        data['sourcey'] = data['screenheight'] - data['screenmargin'] - (data['logoheight'] / 2)
        data['sourcewidth'] = data['screenwidth'] - (data['screenmargin'] * 2) - data['logowidth']
        data['sourceheight'] = data['logoheight']

        # Anchor and position of the debug text
        data['debuganchor'] = gtk.ANCHOR_SE
        data['debugx'] = data['screenwidth'] - data['screenmargin']
        data['debugy'] = data['screenheight'] - data['screenmargin']

        self.set_bounds(0, 0, data['screenwidth'], data['screenheight'])

        self.postroot = goocanvas.Group(parent = self.root,
                                    x = data['postx'], y = data['posty'],
                                    width = data['postwidth'],
                                    height = data['postheight'])

        if DEBUG:
            self.goodebug = goocanvas.Text(parent = self.root, text = 'DEBUG',
                            anchor = data['debuganchor'], x = data['debugx'], 
                            y = data['debugy'], fill_color="white", 
                            font="Sans 8")
            glib.timeout_add_seconds(1, self.renderdebug)

        self.goologo = CustomSvgItem(parent = self.root, handle = logo_handle,
                            x = data['logox'], y = data['logoy'])
        self.goosource = goocanvas.Text(parent = self.root, text = sourcetxt,
                            anchor = data['sourceanchor'], x = data['sourcex'],
                            y = data['sourcey'], width = data['sourcewidth'],
                            height = data['sourceheight'], 
                            fill_color = config['source_color'],
                            font = config['source_font'])
        self.goouser = goocanvas.Text(parent = self.postroot, text = usertxt,
                            anchor = data['authoranchor'], x = data['authorx'],
                            y = data['authory'], width = data['authorwidth'],
                            height = data['authorheight'],
                            fill_color = config['user_color'],
                            font = config['user_font'])
        self.goodate = goocanvas.Text(parent = self.postroot, text = datetxt,
                            anchor = data['dateanchor'], x = data['datex'],
                            y = data['datey'], width = data['datewidth'],
                            height = data['dateheight'],
                            fill_color = config['date_color'],
                            font = config['date_font'])
        self.gootext = goocanvas.Text(parent = self.postroot, text = texttxt,
                            anchor = data['textanchor'], x = data['textx'],
                            y = data['texty'], width = data['textwidth'],
                            height = data['textheight'],
                            fill_color = config['post_color'],
                            font = config['post_font'],
                            wrap = pango.WRAP_WORD_CHAR,
                            ellipsize = pango.ELLIPSIZE_NONE)

        # FIXME: Actually do scale some time!
        #self.postroot.scale(data['postscale'], data['postscale'])

        if hook:
            hook()
        return(False)

    def render(self, item):
        """Render the supplied text onto the canvas items"""
        def swooped(item, stopped, post):
            """Event handler to step through transition animation. If we're
            officially on-screen, we've just swooped out and need to animate
            back in. Otherwise we just swooped back in and are all done"""
            if item.get_property('x') == self.data['postx']:
                dbg('GooRender', 'swooped: off-screen, rendering an item')
                self.render_item(post)
                item.set_property('x', self.data['rightwing'])
                item.animate(-self.data['rightwing'] + self.data['postx'], 0, 1, 0,
                        False, 200, 32, goocanvas.ANIMATE_RESET)
            elif item.get_property('x') == self.data['rightwing']:
                dbg('GooRender', 'swooped: on-screen, disconnected')
                item.set_property('x', self.data['postx'])
                item.disconnect(handler)
            else:
                dbg('GooRender', 'swooped: fail. we are: %s' % item.get_property('x'))

        if item['source'] == '' or self.lifesaver.config['disable_animation']:
            dbg('GooRender', 'Not skipping render_item')
            self.render_item(item)
            return
        else:
            dbg('GooRender', 'Skipping render_item.')

        handler = self.postroot.connect('animation-finished', swooped, item)
        self.postroot.animate(self.data['leftwing'], 0, 1, 0, False, 200, 32,
                goocanvas.ANIMATE_RESET)

    def render_item(self, item):
        """Update the text items"""
        config = self.lifesaver.config
        source = item['source']
        keyword = item['keyword']
        user = item['from_user']
        date = item['created_at']
        text = item['text']
        if keyword != '':
            source = '%s on %s' % (keyword, source)

        self.goosource.set_property('text', source)
        self.goouser.set_property('text', user)
        self.gootext.set_property('text', text)
        self.goodate.set_property('text', date)

    def renderdebug(self):
        """Render debugging items onto the canvas"""
        if self.goodebug:
            self.goodebug.set_property('text', self.counter)
        self.counter += 1
        return(True)
