#!/usr/bin/python
# Lifesaver by Chris Jones <cmsj@tenshu.net>
# GPL v2 only
"""lifesaver.py - Main class"""

import random
import gconf
import gtk
import borg
from util import dbg

class Lifesaver(borg.Borg):
    """Class that implements the top-level things we care about"""

    # Config dictionary. Populate it with defaults just in case
    # gconf fails us
    config = None
    sources = None
    target = None
    gconf = None

    def __init__(self):
        """Class initialiser"""
        borg.Borg.__init__(self)
        self.prepare_attributes()

    def prepare_attributes(self):
        """Initialise anything that isn't already"""
        if not self.config:
            self.config = { 'keywords': ['ubuntu'],
                    'disable_animation': False,
                    'background_color': 'black',
                    'user_font': 'Sans 28',
                    'user_color': 'orange',
                    'date_font': 'Sans 14',
                    'date_color': 'grey',
                    'post_font': 'Sans 28',
                    'post_color': 'white',
                    'source_font': 'Sans 16',
                    'source_color': 'grey',
                    }
        if not self.sources:
            self.sources = []
        if not self.gconf:
            self.gconf = gconf.client_get_default()

    def load_config(self):
        """Load the config from gconf"""
        path = '/apps/lifesaver/'

        for key in self.config.keys():
            if key == 'keywords':
                tmp = self.gconf.get_list(path + key, 'string')
            elif key == 'disable_animation':
                tmp = self.gconf.get_bool(path + key)
            else:
                tmp = self.gconf.get_string(path + key)

            if tmp is not None and tmp != []:
                self.config[key] = tmp

    def set_target(self, target):
        """Store a reference to the target renderer"""
        self.target = target

    def add_source(self, source):
        """Add a source"""
        self.sources.append(source)

    def randomsource(self):
        """Return a random, valid source"""
        tmp = []

        for source in self.sources:
            if source.has_items():
                tmp.append(source)

        if len(tmp) == 0:
            # We don't have any sources with actual data
            raise(ValueError)

        return(tmp[random.randint(0, len(tmp) - 1)])

    def update_sources(self):
        """Update a random source"""
        self.sources[random.randint(0, len(self.sources) - 1)].fetch_data()
        return(True)

    def initial_update(self):
        """Called right after our window is ready"""
        self.initial_item()
        while gtk.events_pending():
            gtk.main_iteration()
        self.update_sources()
        return(False)

    def initial_item(self):
        """Get an initial item rendered"""
        return(self.new_item())

    def normal_item(self):
        """Get a new item from our data"""
        self.new_item()
        return(True)
        
    def new_item(self):
        """Render a single interesting item from a random source"""
        dbg('Lifesaver', 'Getting an item')
        try:
          source = self.randomsource()
        except ValueError:
            from emptysource import EmptySource
            source = EmptySource()

        item = source.get_item()

        if self.target:
            self.target.render(item)
        else:
            dbg('Lifesaver', 'new_item: %s (%s), %s, %s' % (item['from_user'], 
                item['source'], item['created_at'], item['text']))

        if source not in self.sources:
            return(True)
        else:
            return(False)
